using System;
using System.Globalization;
using System.Windows.Forms;
using Microsoft.Win32;

namespace Mintfix.Client
{
    static internal class SettingsManager
    {
        private const string CRegKeyPath = "SOFTWARE\\Mintfix Notification";
        private const string CAutoRunRegPath = @"SOFTWARE\Microsoft\Windows\CurrentVersion\Run";
        private const int DefaultPeriod = 15;


        private static string _DirtyServiceUrl;
        private static int? _DirtyPeriod;
        private static bool? _DirtyAutorun;


        public static event SettingsChangedDelegate OnSettingsChanged;

        private static void OnOnSettingsChanged()
        {
            
            SettingsChangedDelegate handler = OnSettingsChanged;
            if (handler != null) handler();
        }

        public static string GetServiceUrl()
        {
            if (_DirtyServiceUrl == null)
                _DirtyServiceUrl = GetRegistrySetting("ServiceUrl");

            return _DirtyServiceUrl;
        }

        public static int GetPeriod()
        {
            if (!_DirtyPeriod.HasValue)
            {
                var rperiod = GetRegistrySetting("Period");
                if (!String.IsNullOrEmpty(rperiod))
                {
                    int period;
                    if (Int32.TryParse(rperiod, out period))
                    {
                        return period;
                    }
                }

                _DirtyPeriod = DefaultPeriod;
            }

            return _DirtyPeriod.Value;
        }

        public static bool GetAutorun()
        {
            if(!_DirtyAutorun.HasValue)
            {
                _DirtyAutorun = !String.IsNullOrEmpty(GetRegistrySetting("Notification.exe", CAutoRunRegPath));
            }
            return _DirtyAutorun.Value;
        }

        public static void SetPeriod(int period)
        {

            _DirtyPeriod = period;
        }

        public static void SetServiceUrl(string  url)
        {

            _DirtyServiceUrl = url;
        }

        public static void SetAutorun(bool autorunEnabled)
        {
            _DirtyAutorun = autorunEnabled;


        }

        public static void Save()
        {
            SaveRegistrySetting("Period", GetPeriod().ToString(CultureInfo.InvariantCulture));
            SaveRegistrySetting("ServiceUrl", GetServiceUrl());
            if (GetAutorun())
            {
                SaveRegistrySetting("Notification.exe", Application.ExecutablePath, CAutoRunRegPath);
            }
            else
            {
                DeleteRegistryKey("Notification.exe", CAutoRunRegPath);
            }

            OnOnSettingsChanged();
        }

        private static void DeleteRegistryKey(string keyName, string regKeyPath = CRegKeyPath)
        {
            RegistryKey regKey = Registry.CurrentUser.OpenSubKey(regKeyPath, true);
            if (regKey != null && regKey.GetValue(keyName) != null)
            {
                regKey.DeleteValue(keyName);
                regKey.Close();
            }

        }

        private static void SaveRegistrySetting(string settingName, string settingVal, string regKeyPath = CRegKeyPath)
        {

            if (!String.IsNullOrEmpty(settingVal))
            {
                var regKey = Registry.CurrentUser.CreateSubKey(regKeyPath);
                if (regKey != null)
                {
                    regKey.SetValue(settingName, settingVal);
                    regKey.Close();
                }
            }
        }

        private static string GetRegistrySetting(string settingName, string regKeyPath = CRegKeyPath)
        {
            string settingVal = String.Empty;
            RegistryKey regKey = Registry.CurrentUser.OpenSubKey(regKeyPath , true);
            if (regKey != null)
            {
                settingVal =  regKey.GetValue( settingName, String.Empty ).ToString();	
				
                regKey.Close();
            }
            return settingVal;
        }
    }
}